// Persistence of Vision Ray Tracer Scene Description File
// File: showoff.pov
// Vers: 3.6
// Desc: Parsys Example scene 
// Date: 12.04.04 (dd.mm.yy)
// Auth: Tim Nikias Wenclawiak - www.NoLights.de
// Last Update: 22.10.06 (dd.mm.yy)

//Required Files
// particles.inc - Tim Nikias' Particle System

//Description
// This file shows off most of the effects that the particle system is capable of.
// A multitude of groups with different rules, particles may seed new particles,
// effects may be dependant on the object a particle hits and is very flexible due
// to the macro-approach.
// Note that the rendering times on this one are quite high, there's media and
// refraction present. At the time of writing this note, the resulting animation
// can be found on my website, won't vow however that it'll be there forever. ;-)

//Commandline-Options: 25fps for 6 seconds:
//+KFI1 +KFF150

// Parsys - I/O Particle System
#include "parsys.inc"
#declare Animation_Time = 6; //Time of Animation in Seconds
#declare PerFrameRandom = seed(frame_number); //different seed per frame

//Particles per Frame
#declare FirePerFrame = 9;
#declare WaterPerFrame = 3;
//Smoke only needs one particle every few frames
#declare SmokeEveryXFrame = 4;

#declare Fire_Switch = on;
#declare Water_Switch = on;
#declare Smoke_Switch = on;

#default{finish{ambient 0}}

global_settings{
  max_trace_level 50
  assumed_gamma 1
}

//Excerpt from my Look_At-Macros
/******************************/
//Look_At
//Parameters:
// 1. Target-Position
//Returns:
// 1. Rotation-Vector
//Notes:
// Assumes object at <0,0,0> facing -z, y=sky_vector
#macro Look_At(Target)
 #if (Target.y!=0)
  #local X_Rot_Float = degrees(atan2(Target.y,vlength(Target*<1,0,1>))); #else
  #local X_Rot_Float = 0; #end
 #if (Target.x!=0)
  #local Y_Rot_Float = degrees(atan2(-Target.x,-Target.z)); #else
  #if (Target.z<0)
   #local Y_Rot_Float = 0; #else
   #local Y_Rot_Float =180; #end
  #end
 #local Result=<X_Rot_Float, Y_Rot_Float, 0>;
 (Result)
#end

camera{
  location <-.6,3,-7>
  look_at <-.6,2,0>
}

//Three-Point-Lighting
light_source{
  <400,300,-120> rgb 1
  }
light_source{
  <-200,300,-400> rgb <.1,.1,.2>
  shadowless
  }
light_source{
  <100,100,400> rgb <.16,.16,.2>
  shadowless
  }

//A simple, blue sky_sphere
sky_sphere{
  pigment{
    gradient y
    color_map{
      [0 rgb 1]
      [.35 rgb .5]
      [.7 rgb <.5,.5,1>*.5]
      [1 rgb z*.2]
    }
    scale 2 translate -1
  }
}

//Background Mountainrange
#declare Flat_Pig = pigment{bozo poly_wave 2 turbulence .34 lambda 2.2 omega .6 color_map{[0 rgb 0][1 rgb .07]}}
#declare High_Pig = pigment{bozo turbulence .34 lambda 2.2 omega .6 color_map{[0 rgb .5][1 rgb 1]}}
#declare HF_Pigment = pigment{gradient y poly_wave .5 phase -.01 pigment_map{[0 High_Pig scale .1][1 Flat_Pig]}}
#declare MountainRange =
height_field{ 
  function 300,300 {pigment{HF_Pigment}}
  translate <-.5,0,0>
  scale <1,1,1>*<35,9,20>
  translate <0,-.65,-3>
  }

//Textures for the Mountainrange
#declare Dirt_Pigment =
  pigment{
    bozo color_map{
      [0 rgb <.13,.07,.02>]
      [1 rgb <.1,.05,.01>]
    }
    scale .3
  }
#declare Grass_Pigment =
  pigment{
    bozo color_map{
      [0 rgb <.03,.27,.03>]
      [1 rgb <.03,.35,.05>]
    }
    scale .07
  }
#declare Ground_Texture =
  texture{
    pigment{
      slope{y,.5,1}
      poly_wave 2 pigment_map{
        [0 Dirt_Pigment]
        [.3 Dirt_Pigment]
        [.4 Grass_Pigment]
        [1 Grass_Pigment]
      }
    }
    finish{
      diffuse .8
      brilliance 1.5
    }
  }

#declare Ground =
union{
  object{MountainRange}
  plane{y,-.4}
  texture{Ground_Texture}
  hollow
}

#declare Water_Outlet =
  union{
    torus{.35,.05
      rotate x*90
      rotate Look_At(vnormalize(<1.25,9,0>))
      translate <-2.5,0,0>+vnormalize(<1.25,9,0>)
      }
    cone{<-2.5,0,0>-vnormalize(<1.25,9,0>)*.25,.3,<-2.5,0,0>+vnormalize(<1.25,9,0>)*1,.4 open}
    cone{<-2.5,0,0>-vnormalize(<1.25,9,0>)*.25,.275,<-2.5,0,0>+vnormalize(<1.25,9,0>)*1,.3 open}
    pigment{rgb .5}
    finish{
      specular .8 roughness .02
      brilliance 5
      reflection{0,.9}
    }
  }

object{Ground}
object{Water_Outlet}

//Timer for the "Burning Thing"
#declare Fire_Clock=1;
#if (frame_number <= 15)
  #declare Fire_Clock = max((frame_number-1),0)/15;
#end
#if (frame_number >= 70 & frame_number <=100)
  #declare Fire_Clock = 1-(frame_number-70)/30;
#end
#if (frame_number > 100)
  #declare Fire_Clock = 0;
#end

//Animated Texture for the "Burning Thing"
#declare Fire_Texture =
  texture{
    pigment{crackle
      color_map{
        [0-Fire_Clock*.2 rgb .1]
        [.5-Fire_Clock*.45 rgb .2]
        [.7-Fire_Clock rgb <1,.3,0>]
        [1-Fire_Clock rgb <1,.4+Fire_Clock*.6,Fire_Clock*.4>]
      }
      scale .2 turbulence .6 lambda 3 omega .65 octaves 4
    }
    finish{ambient sqrt(Fire_Clock)}
  }
  texture{
    pigment{rgb .06 transmit Fire_Clock}
    normal{crackle .06 poly_wave 2 no_bump_scale scale .1}
    finish{
      specular .07 roughness .12
      }
  }

//Simple Shadow-setup to have control over the translucency of the
//"Burning Thing"'s shadow.
#declare Burning_Thing = sphere{y*.5,.5}
object{Burning_Thing texture{Fire_Texture} no_shadow}
object{Burning_Thing pigment{rgbt <0,0,0,Fire_Clock>} no_image no_reflection}

//Burning-Thing Glow
#if (Fire_Clock!=0)
light_source{y*.5, <pow(Fire_Clock,2),pow(Fire_Clock,3),pow(Fire_Clock,4)>*2
  fade_power 2 fade_distance 1.5
  media_interaction off
}
sphere{0,1 hollow
  pigment{rgbt 1}
  interior{
    media{
      emission Fire_Clock
      method 2 samples 1,1 intervals 5
      density{onion frequency 11 color_map{[0 rgb 1][1 rgb 0]}}
    }
  }
  no_shadow
  scale .4999+.05*pow(Fire_Clock,4)
  translate y*.5
}
#end

//Distance from which trace() are shot at the object to place
// fire-particles
#declare Burning_Thing_Size=2;

/** Visualization Macros **/
//The different groups get their own Visualization macros according
//to their use. "water"- and "droplet"-Particles share the same macro,
//as they are both used within one blob{} and "droplets" are actually
//meant to be smaller parts of "water"-Particles.

//Visualisation Macro for the Fire-Particles
// Simple color-changes as the particle ages, additionally it grows
// fainter after some time to vanish.
#macro Fire_Particle(Data)
  #local Age=Data[0].x;
  #local Intensity=4;
  sphere{0,1 hollow
    pigment{rgbt 1}
    interior{
      media{
        method 2 intervals 5 samples 1,1
        emission max((1-Age)*2,0)*Intensity
        absorption max(1-Age,0)*Intensity
        density{
          spherical color_map{[0 rgb 0][1 rgb 1]}
          scale .8
          turbulence .24 lambda 2.5 omega .6 octaves 7
          rotate Data[4]+Data[4]*Age

        }
        density{
          bozo
          color_map{[0 rgb 0][.5+min(Age,.25) rgb x+y*(.5-min(Age,.5))][1 rgb <1,.75-min(Age,.5),0>]}
          scale .5
          translate Data[2]*.5
        }
      }
    }
    no_shadow
    scale .13*min(Data[0].x*10,1)+.125*min(Data[0].x*2,.75)
    translate Data[2]
  }
#end

//Visualisation Macro for the Water-Particles
// Data[3] is the velocity of the particle, I stretch the particle
// along the Z-Axis a little depending on the velocity. Afterwards,
// I rotate the particle to align with the velocity's direction
// using my Look_At-Macro provided at the beginning of the scene.
// Note that these are blob-components, hence twice the size and a
// strength-value of 1: The blob threshold is set to .5, so a particle
// need double the radius to appear to be the correct size when
// bouncing off of objects.
#macro Water_Particle(Data)
  #local _Rot = Look_At(Data[3]);
  #local Speed= min(max(vlength(Data[3]),0),5)/5;
  sphere{0,Data[1].y*2.1,1
    scale <1,1,1>+<-.2,-.2,.8>*Speed
    rotate _Rot
    translate Data[2]
    }
#end

//Visualisation Macro for the Smoke
// A smoke-particle grows in size as it ages and then gets
// fainter and fainter until it vanishes.
#macro Smoke_Particle(Data)
  #local Age=Data[0].x;
  #local Int=Data[5].x;
  sphere{0,1 hollow
    pigment{rgbt 1}
    interior{
      media{
        method 2 intervals 5 samples 1,1
        emission max((1-Age)*2,0)*.1*(1-Int)
        scattering{1,1.15*(1-Int)*(1-Age)}
        density{
          spherical color_map{[0 rgb 0][.65 rgb 1][1 rgb 1]}
          scale .8
          turbulence .24 lambda 2.5 omega .6 octaves 7
          rotate (Data[4]+Data[4]*Age)*360
        }
        density{
          bozo
          color_map{[0 rgb .7][1 rgb 1]}
          scale .5
          translate Data[2]*.5
        }
      }
    }
    scale 1*(Data[1].y+.5*min(Data[0].x,1)+.3*min(Data[0].x*10,1))
    translate Data[2]
  }
#end


/** Impact Effect **/
//The main "water"-Particle will, once they hit a surface, *explode* into several
//smaller "droplet"-Particles. Thus, the initial stream is created with less
//particles and can get simulated faster, until more detail is needed for the
//splashing effect.
#macro CreateDroplets(_Data,_ImpactSec,_ImpactPos,_ImpactNormal)
  //Generate Rotation to match Direciton of Velocity
  #local _Rot = Look_At(_ImpactNormal);
  #local _A=0;
  #while (_A<8)
    //Generate new Velocity that roughly points downwards
    #local _NewVelocity = vrotate(vrotate(<1,0,0>,z*360*rand(PerFrameRandom)),_Rot);
    #while (_NewVelocity.y > .3)
      #local _NewVelocity = vrotate(vrotate(<1,0,0>,z*360*rand(PerFrameRandom)),_Rot);
    #end
    #local _NewSpeed =(min(vlength(_Data[3])-3,0)*rand(PerFrameRandom)+3)*.7;
    #local _NewRadius = .03+.03*rand(PerFrameRandom);
    //Add particle
    Particle_Add(
      "droplet",
      array[4]{
        <0,8,_ImpactSec>,
        <10,_NewRadius,0>,
        _ImpactPos+_ImpactNormal*_NewRadius*1.1,
        _NewVelocity*_NewSpeed,
      }
    )
    #local _A=_A+1;
  #end
#end

/** Setting the Groups **/
//Note that I have to add all effects to each group seperately. E.g.,
//if I define wind for one particle-group, I have to define it again
//for a different group.
//Note that I'm using similiar TurbWind-Settings for the "fire" and the
//"smoke" group. This is to simulate a slightly different behaviour.

//Fire-Group
ParticleGroup_Begin("fire",<1/25,1/25,0>)
ParticleGroup_TurbWind(
  2,.6,9,.1, //Turbulence Settings
  <0,10,0>*clock, //Modifies position of field
  20, //Multiplier for the speeds generated by turbulence
  .1)
ParticleGroup_LinearWind(<0,1,0>,5,.2)
ParticleGroup_Object("Ground",1,1,0,"off","off")
ParticleGroup_Object("Water_Outlet",0,1,0,"off","off")
ParticleGroup_Object("Burning_Thing",0,1,0,"off","off")
ParticleGroup_End()

//Water-Group
ParticleGroup_Begin("water",<1/50,1/25,.25>)
ParticleGroup_Gravity(9.81,<0,-1,0>)
ParticleGroup_Object("Ground",0,.9,2,"off","CreateDroplets")
ParticleGroup_Object("Water_Outlet",0,.9,2,"off","CreateDroplets")
ParticleGroup_Object("Burning_Thing",0,.9,2,"off","CreateDroplets")
ParticleGroup_End()

//Droplet-Group
ParticleGroup_Begin("droplet",<1/75,1/50,.1>)
ParticleGroup_Gravity(9.81,<0,-1,0>)
ParticleGroup_Damping(.85)
ParticleGroup_Object("Ground",0,.8,0,"off","off")
ParticleGroup_Object("Water_Outlet",0,.8,0,"off","off")
ParticleGroup_Object("Burning_Thing",0,.8,0,"off","off")
ParticleGroup_End()

//Smoke-Group
ParticleGroup_Begin("smoke",<1/25,1/25,.1>)
ParticleGroup_TurbWind(
  2,.6,9,.1, //Turbulence Settings
  12+<0,10,0>*clock, //Modifies position of field
  20, //Multiplier for the speeds generated by turbulence
  .2)
ParticleGroup_LinearWind(<0,1,0>,5,.2)
ParticleGroup_End()

/** Adding Particles into the Simulation **/
//The different particles are added at different intervals and
//in different quantities. Note that I do not place any "droplet"-
//Particles, as those are created when a "water"-Particle has an
//impact.

//Fire-Particles
#if (frame_number < 70 & Fire_Switch)
  #local A=0;
  #while (A<FirePerFrame)
    #declare OPosition =vnormalize(<rand(PerFrameRandom),rand(PerFrameRandom),rand(PerFrameRandom)>*2-1)*Burning_Thing_Size; 
    #declare NormalCheck = <0,0,0>;
    #declare Position = trace(Burning_Thing,OPosition,-OPosition,NormalCheck)+OPosition*.01;
    #while (!vlength(NormalCheck) | Position.y < .1)
      #declare OPosition =vnormalize(<rand(PerFrameRandom),rand(PerFrameRandom),rand(PerFrameRandom)>*2-1)*Burning_Thing_Size; 
      #declare Position = trace(Burning_Thing,OPosition,-OPosition,NormalCheck)+OPosition*.01;
    #end
    #declare Rotation =<rand(PerFrameRandom),rand(PerFrameRandom),rand(PerFrameRandom)>*360; 
    Particle_Add("fire",
    array[5]{
    <0,1,clock*Animation_Time+clock_delta*Animation_Time*rand(PerFrameRandom)>,
    <0,0,0>,
    Position,
    OPosition*(.25+.25*rand(PerFrameRandom))+y*(.5+2*rand(PerFrameRandom)),
    Rotation,
    })
    #local A=A+1;
  #end
#end

//Water-Particles
#if (frame_number > 25 & frame_number < 75 & Water_Switch)
  #local A=0;
  #while (A<WaterPerFrame)
    #declare Position =vnormalize(<rand(PerFrameRandom),rand(PerFrameRandom),rand(PerFrameRandom)>*2-1); 
    Particle_Add("water",
    array[4]{
    <0,6,clock*Animation_Time+clock_delta*Animation_Time*rand(PerFrameRandom)>,
    <0,.09,0>,
    Position*.1+<-2.5,.5,0>,
    <1.4,9,0>+Position*.5*rand(PerFrameRandom),
    })
    #local A=A+1;
  #end
#end

//Smoke-Particles
#if (frame_number >= 70 & frame_number <= 100 & mod(frame_number,SmokeEveryXFrame)=0 & Smoke_Switch)
  #declare IClock = (frame_number-70)/30;
  #local A=0;
  #while (A<1)
    #declare Position = y*.5; 
    Particle_Add("smoke",
    array[6]{
    <0,1,clock*Animation_Time+clock_delta*Animation_Time*rand(PerFrameRandom)>,
    <0,.45,0>,
    Position,
    <0,3,0>+y*rand(PerFrameRandom),
    <rand(PerFrameRandom),rand(PerFrameRandom),rand(PerFrameRandom)>,
    <IClock,0,0> //Intensity
    })
    #local A=A+1;
  #end
#end

/** Run simulation **/
Parsys_Run()


/** Visualizes Particles with a Macro **/
//Here, we have to notice that the "water"- and "droplet"-Particles
//make use of a macro which returns sphere's with the blob-related
//strength-value. Thus, the Show_Particles-Macro has to be called
//inside the blob, otherwise, POV-Ray would return an error, as blob-
//components may not stand by themselves.
Show_Particles("fire","Fire_Particle")
Show_Particles("smoke","Smoke_Particle")
blob{
  threshold .5
  Show_Particles("water","Water_Particle")
  Show_Particles("droplet","Water_Particle")
  sphere{-y,.1,.1}
  pigment{rgb <.4,.4,1> transmit .9}
  interior{ior 1.44 fade_power 2 fade_distance .5}
  finish{
    specular .8 roughness .01
    reflection{0,1 fresnel}
    conserve_energy
    }
}
